@@ -28,9 +28,8 @@ exe_cli = [os.path.join(rootdir, 'src', 'netplan.script')]
 if shutil.which('python3-coverage'):
     exe_cli = ['python3-coverage', 'run', '--append', '--'] + exe_cli
 
-# Make sure we can import our development netplan.
-os.environ.update({'PYTHONPATH': '.'})
-NETPLAN_DBUS_CMD = os.path.join(os.path.dirname(__file__), "..", "..", "netplan-dbus")
+NETPLAN_DBUS_CMD = os.environ.get('NETPLAN_DBUS_CMD',
+                                  os.path.join(os.path.dirname(__file__), "..", "..", "netplan-dbus"))
 
 
 class TestNetplanDBus(unittest.TestCase):
