@@ -65,17 +65,13 @@ class IntegrationTestsBase(unittest.TestCase):
         # configured. It should be running all the time, independently of netplan
         os.makedirs('/etc/systemd/network', exist_ok=True)
         with open('/etc/systemd/network/20-wired.network', 'w') as f:
-            f.write('[Match]\nName=eth0 en*\n\n[Network]\nDHCP=ipv4')
-
-        # ensure NM can manage our fake eths
-        os.makedirs('/run/udev/rules.d', exist_ok=True)
-        with open('/run/udev/rules.d/99-nm-veth-test.rules', 'w') as f:
-            f.write('ENV{ID_NET_DRIVER}=="veth", ENV{INTERFACE}=="eth42|eth43|iface1|iface2", ENV{NM_UNMANAGED}="0"\n')
-        subprocess.check_call(['udevadm', 'control', '--reload'])
+            f.write('[Match]\nName=eth0 en*\n\n[Network]\nDHCP=yes\nKeepConfiguration=yes')
 
+        # ensure NM doesn't interfere with our test backend (fake eth endpoints & mgmt network)
         os.makedirs('/etc/NetworkManager/conf.d', exist_ok=True)
         with open('/etc/NetworkManager/conf.d/99-test-ignore.conf', 'w') as f:
-            f.write('[keyfile]\nunmanaged-devices+=interface-name:eth0,interface-name:en*,interface-name:veth42,interface-name:veth43')
+            f.write('''[keyfile]
+unmanaged-devices+=interface-name:eth0,interface-name:en*,interface-name:veth42,interface-name:veth43''')
         subprocess.check_call(['netplan', 'apply'])
         subprocess.call(['/lib/systemd/systemd-networkd-wait-online', '--quiet', '--timeout=30'])
 
@@ -85,10 +81,6 @@ class IntegrationTestsBase(unittest.TestCase):
             os.remove('/run/NetworkManager/conf.d/test-blacklist.conf')
         except FileNotFoundError:
             pass
-        try:
-            os.remove('/run/udev/rules.d/99-nm-veth-test.rules')
-        except FileNotFoundError:
-            pass
 
     def tearDown(self):
         subprocess.call(['systemctl', 'stop', 'NetworkManager', 'systemd-networkd', 'netplan-wpa-*',
@@ -104,7 +96,12 @@ class IntegrationTestsBase(unittest.TestCase):
             os.remove(f)
         for f in glob.glob('/run/systemd/system/**/netplan-*'):
             os.remove(f)
+        for f in glob.glob('/run/udev/rules.d/*netplan*'):
+            os.remove(f)
         subprocess.call(['systemctl', 'daemon-reload'])
+        subprocess.call(['udevadm', 'control', '--reload'])
+        subprocess.call(['udevadm', 'trigger', '--attr-match=subsystem=net'])
+        subprocess.call(['udevadm', 'settle'])
         try:
             os.remove('/run/systemd/generator/netplan.stamp')
         except FileNotFoundError:
@@ -275,7 +272,7 @@ class IntegrationTestsBase(unittest.TestCase):
     def assert_iface(self, iface, expected_ip_a=None, unexpected_ip_a=None):
         '''Assert that client interface has been created'''
 
-        out = subprocess.check_output(['ip', 'a', 'show', 'dev', iface],
+        out = subprocess.check_output(['ip', '-d', 'a', 'show', 'dev', iface],
                                       universal_newlines=True)
         if expected_ip_a:
             for r in expected_ip_a:
@@ -300,7 +297,12 @@ class IntegrationTestsBase(unittest.TestCase):
         cmd = ['netplan', 'apply']
         if state_dir:
             cmd = cmd + ['--state', state_dir]
-        out = subprocess.check_output(cmd, stderr=subprocess.STDOUT, universal_newlines=True)
+        out = ''
+        try:
+            out = subprocess.check_output(cmd, stderr=subprocess.STDOUT, universal_newlines=True)
+        except subprocess.CalledProcessError as e:
+            self.assertTrue(False, 'netplan apply failed: {}'.format(e.output))
+
         if 'Run \'systemctl daemon-reload\' to reload units.' in out:
             self.fail('systemd units changed without reload')
         # start NM so that we can verify that it does not manage anything
