@@ -25,8 +25,6 @@ from .base import TestKeyfileBase
 
 rootdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 exe_cli = os.path.join(rootdir, 'src', 'netplan.script')
-# Make sure we can import our development netplan.
-os.environ.update({'PYTHONPATH': '.'})
 
 lib = ctypes.CDLL(ctypes.util.find_library('netplan'))
 lib.netplan_get_id_from_nm_filename.restype = ctypes.c_char_p
@@ -287,6 +285,7 @@ address1=1:2:3::9/128
 gateway=6:6::6
 route1=dead:beef::1/128,2001:1234::2
 route1_options=unknown=invalid,
+route2=4:5:6:7:8:9:0:1/63,,5
 
 [proxy]
 '''.format(UUID))
@@ -306,10 +305,6 @@ route1_options=unknown=invalid,
         - 9.8.7.6
         - 5.4.3.2
         - dead:beef::2
-        search:
-        - foo.local
-        - bar.remote
-        - bar.local
       gateway4: 6.6.6.6
       gateway6: 6:6::6
       ipv6-address-generation: "stable-privacy"
@@ -328,6 +323,9 @@ route1_options=unknown=invalid,
         via: "4.4.4.4"
       - to: "dead:beef::1/128"
         via: "2001:1234::2"
+      - scope: "link"
+        metric: 5
+        to: "4:5:6:7:8:9:0:1/63"
       wakeonlan: true
       networkmanager:
         uuid: "{}"
@@ -600,7 +598,8 @@ mode=ap'''.format(UUID))
         self._template_keyfile_type_wifi('infrastructure', 'mesh')
 
     def test_keyfile_type_wifi_missing_ssid(self):
-        err = self.generate_from_keyfile('''[connection]\ntype=wifi\nuuid={}\nid=myid with spaces'''.format(UUID), expect_fail=True)
+        err = self.generate_from_keyfile('''[connection]\ntype=wifi\nuuid={}\nid=myid with spaces'''
+                                         .format(UUID), expect_fail=True)
         self.assertFalse(os.path.isfile(os.path.join(self.confdir, '90-NM-{}.yaml'.format(UUID))))
         self.assertIn('netplan: Keyfile: cannot find SSID for WiFi connection', err)
 
@@ -1093,8 +1092,6 @@ route3=4:5:6:7:8:9:0:1/63,::,5
 route4=5:6:7:8:9:0:1:2/62
 
 [proxy]
-
-
 '''.format(UUID))
         self.assert_netplan({UUID: '''network:
   version: 2
@@ -1114,8 +1111,6 @@ route4=5:6:7:8:9:0:1:2/62
         - 4.2.2.2
         - 1::cafe
         - 2::cafe
-        search:
-        - wallaceandgromit.com
       ipv6-address-generation: "stable-privacy"
       mtu: 900
       routes:
@@ -1242,3 +1237,117 @@ method=auto
         passthrough:
           ipv6.ip6-privacy: "-1"
 '''.format(UUID, UUID)})
+
+    def test_keyfile_wpa3_sae(self):
+        self.generate_from_keyfile('''[connection]
+id=test2
+uuid={}
+type=wifi
+interface-name=wlan0
+
+[wifi]
+mode=infrastructure
+ssid=ubuntu-wpa2-wpa3-mixed
+
+[wifi-security]
+key-mgmt=sae
+psk=test1234
+
+[ipv4]
+method=auto
+
+[ipv6]
+addr-gen-mode=stable-privacy
+method=auto
+
+[proxy]
+'''.format(UUID))
+        self.assert_netplan({UUID: '''network:
+  version: 2
+  wifis:
+    NM-{}:
+      renderer: NetworkManager
+      match:
+        name: "wlan0"
+      dhcp4: true
+      dhcp6: true
+      ipv6-address-generation: "stable-privacy"
+      access-points:
+        "ubuntu-wpa2-wpa3-mixed":
+          auth:
+            key-management: "none"
+            password: "test1234"
+          networkmanager:
+            uuid: "ff9d6ebc-226d-4f82-a485-b7ff83b9607f"
+            name: "test2"
+            passthrough:
+              wifi-security.key-mgmt: "sae"
+              ipv6.ip6-privacy: "-1"
+              proxy._: ""
+      networkmanager:
+        uuid: "{}"
+        name: "test2"
+'''.format(UUID, UUID)})
+
+    def test_keyfile_dns_search_ip4_ip6_conflict(self):
+        self.generate_from_keyfile('''[connection]
+id=Work Wired
+type=ethernet
+uuid={}
+autoconnect=false
+timestamp=305419896
+
+[ethernet]
+wake-on-lan=1
+mac-address=99:88:77:66:55:44
+mtu=900
+
+[ipv4]
+method=manual
+address1=192.168.0.5/24,192.168.0.1
+address2=1.2.3.4/8
+dns=4.2.2.1;4.2.2.2;
+
+[ipv6]
+method=manual
+address1=abcd::beef/64
+address2=dcba::beef/56
+addr-gen-mode=1
+dns=1::cafe;2::cafe;
+dns-search=wallaceandgromit.com;
+
+[proxy]\n'''.format(UUID))
+        self.assert_netplan({UUID: '''network:
+  version: 2
+  ethernets:
+    NM-{}:
+      renderer: NetworkManager
+      match:
+        macaddress: "99:88:77:66:55:44"
+      addresses:
+      - "192.168.0.5/24"
+      - "1.2.3.4/8"
+      - "abcd::beef/64"
+      - "dcba::beef/56"
+      nameservers:
+        addresses:
+        - 4.2.2.1
+        - 4.2.2.2
+        - 1::cafe
+        - 2::cafe
+      mtu: 900
+      wakeonlan: true
+      networkmanager:
+        uuid: "{}"
+        name: "Work Wired"
+        passthrough:
+          connection.autoconnect: "false"
+          connection.timestamp: "305419896"
+          ethernet.wake-on-lan: "1"
+          ipv4.method: "manual"
+          ipv4.address1: "192.168.0.5/24,192.168.0.1"
+          ipv6.addr-gen-mode: "1"
+          ipv6.dns-search: "wallaceandgromit.com;"
+          ipv6.ip6-privacy: "-1"
+          proxy._: ""
+'''.format(UUID, UUID)})
